<?php
/*****
* Web Browser display class.
* 99% of what is displayed to the browser (with the exception of basic login and error messages) is parsed and 
* printed here.  Nothing should be printed directly to the browser from other classes except error messages 
* from the AUTH class and MANIFEST class.  Collect browser input from CLASSes here and print in one go AFTER it's all 
* been checked for errors.  This ensures that error messages and bailing out of the scripts are accomplished BEFORE 
* any other printout.  The generic variable $pString is passed around between classes to accomplish this.
*****/
class DISPLAY
{
// Constructor
	function DISPLAY($db, $auth, $vars, $displayType)
	{
		$this->db = $db;
		$this->vars = $vars;
		$this->auth = $auth;
		$this->subMenuString = '';
		$this->resources = 0;
		$this->quotes = 0;
		$this->musings = 0;
		$this->version = "0.9.6";
		include_once("classes/SESSION.php");
		$this->session = new SESSION();
		if($displayType == 'bailout')
		{
			$this->bailout(FALSE);
			return;
		}
		if($displayType == 'bailout2')
		{
			$this->bailout(TRUE);
			return;
		}
		$this->checkConfig();
// Framing DIV
		print "<DIV CLASS=\"frame\">";
		$this->head();
// help must be dealt with before we print out menu or submenu options
		if($displayType == 'help')
		{
			include_once("classes/HELP.php");
			$this->help = new HELP();
			print $this->help->main();
			$this->footer();
			return;
		}
		print "<DIV CLASS=\"menu\">";
		$this->menu();
		if(!method_exists($this, $displayType))
			$this->auth->gracefulExit(2, "Incorrect or missing parameters.");
		$this->subMenuFormat($displayType);
		$this->submenu();
// Close 'menu' DIV
		print "</DIV>";
		print "<DIV CLASS=\"clear\"></DIV>";
		print "<DIV class=\"mainFrame\">";
		print "<DIV class=\"main\">";
// Point at which 99% browser input is printed out.
		print $this->$displayType();
		$this->footer();
	}
// bailout method prints the menus and any error message.  Method only called from AUTH::gracefulExit
	function bailout($printSubMenu)
	{
// $this->head() will already have been printed.
		if($printSubMenu)
		{
			$this->subMenuFormat('bailout');
			$this->submenu();
			print "<DIV CLASS=\"clear\"></DIV>";
			print "</DIV>";
			print "<DIV class=\"mainFrame\">";
			print "<DIV class=\"main\">";
		}
		if(isset($this->vars['message']) && $this->vars['message'])
			print "<DIV class=\"warning\">ERROR: " . $this->vars['message'] . "</DIV>";
		$this->footer();
	}
// print browser window headers common to all windows
	function head()
	{
		print quotemeta('<HTML><HEAD><META HTTP-EQUIV="EXPIRES" CONTENT="0">');
		print quotemeta('<META HTTP-EQUIV="Content-Type" content="text/html; charset=iso-8859-1">');
		print "<link rel=\"stylesheet\" href=\"$this->style\" type=\"text/css\" media=\"screen\">";
		if(!isset($this->vars['action']) || !$this->vars["action"])
		{
			print "<TITLE>$this->title</TITLE>";
		}
		else
		{
			print "<TITLE>$this->title: " . $this->vars["action"] . "</TITLE>";
		}
		print quotemeta('</HEAD><BODY>');
		print "<DIV CLASS=\"header\">";
		print "<DIV CLASS=\"headerTitle\"><STRONG>$this->title</STRONG></DIV>";
		print "<DIV CLASS=\"headerDescription\">$this->description</DIV>";
		print "<DIV CLASS=\"headerImage\"><IMG SRC=\"$this->image\" 
			HEIGHT=\"$this->imageHeight\" WIDTH=\"$this->imageWidth\"ALT=\"logo\"></DIV>";
		print "<DIV CLASS=\"clear\"></DIV>";
		print "</DIV>";
	}
// gather details for summary report of database
	function gatherReport()
	{
		$sql = $this->db->select('summary', "total_resources, total_quotes, total_musings");
		$row = $this->db->fetchAssoc($this->db->query($sql));
		$this->resources = $row['total_resources'];
		$this->quotes = $row['total_quotes'];
		$this->musings = $row['total_musings'];
	}
// display menu.  Must check session.  2 possibilities: nothing set (so just read permission) or 
// 'write' is TRUE - so user has full read/write/edit permissions.
// Display options accordingly.
	function menu()
	{
		global $PHP_SELF;
		$admin = "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . "?action=admin\">Admin</A>";
		print "<DIV CLASS=\"mainmenu\">";
		if(!isset($this->vars['action']) || $this->vars['action'] == 'front')
		{
			print "Home";
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=resource\">Resources</A>";
			print $admin;
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=help\" TARGET=\"_help\">Help</A>";
		}
		else if($this->vars['action'] == 'resource')
		{
			print "<A HREF=\"" . $PHP_SELF . "?action=front\">Home</A>";
			print "&nbsp;&nbsp;|&nbsp;&nbsp;Resources";
			print $admin;
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=help\" TARGET=\"_help\">Help</A>";
		}
		else if($this->vars['action'] == 'admin')
		{
			print "<A HREF=\"" . $PHP_SELF . "?action=front\">Home</A>";
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=resource\">Resources</A>";
			print $admin = "&nbsp;&nbsp;|&nbsp;&nbsp;Admin";
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=help\" TARGET=\"_help\">Help</A>";
		}
		else
		{
			print "<A HREF=\"" . $PHP_SELF . "?action=front\">Home</A>";
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=resource\">Resources</A>";
			print $admin;
			print "&nbsp;&nbsp;|&nbsp;&nbsp;
				<A HREF=\"" . $PHP_SELF . "?action=help\" TARGET=\"_help\">Help</A>";
		}
		print "</DIV>";
	}
// Display the front page
	function front()
	{
		$pString = "<DIV CLASS=\"intro\">" . $this->frontpage . "</DIV>";
		$pString .= "<DIV CLASS=\"clear\"><R></DIV>";
// If resources exist, print latest action
		if($this->resources)
		{
			include_once("classes/CONFIG.php");
			$config = new CONFIG();
			$config->commonSql('all');
// + 0 on timestamp is for MySQL compatibility with v4.0 and up.  Ensures that whatever v.MySQL used, timestamp 
// is always returned as a numeric string.
// (Updating a resource table row automatically updates the timestamp column).
			$config->stmt .= ", UNIX_TIMESTAMP(timestamp + 0) AS timestamp, 
				num_quotes, num_musings, notes";
			$config->condition .= " LEFT JOIN resource_notes ON resource.id=resource_notes.id 
				ORDER BY timestamp DESC LIMIT 1";
			$sql = $this->db->select('resource', $config->stmt, $config->condition);
			$sth = $this->db->query($sql);
			$row = $this->db->fetchAssoc($sth);
			$date = date("jS F Y", $row['timestamp']);
			$pString .= "Last addition/edit ($date)<BR>";
			$pString .= $this->displayResource($row, 0);
		}
// List groups/categories.
		global $WIKINDX_GROUPS;
		$groups = $WIKINDX_GROUPS;
		natcasesort($groups);
		if(sizeof($groups) > 1)
		{
			$pString .= "The administrator of this WIKINDX has initialised the following groups:
			<DIV class=\"list\"><UL>";
			foreach($groups as $group)
				$pString .= "<LI><B>$group</B>";
			$pString .= "</UL></DIV>";
			$pString .= "<DIV CLASS=\"clear\"></DIV>";
		}
		global $WIKINDX_CONTACT;
		if(isset($WIKINDX_CONTACT))
			$pString .= $WIKINDX_CONTACT;
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=front');
		$this->session->setVar('previousLinkName', 'Home');
		return $pString;
	}
// Administration options
	function admin()
	{
		include_once("classes/ADMIN.php");
		$admin = new ADMIN($this->db, $this->auth, $this->vars);
		$admin->authorize();
		$pString = "<DIV CLASS=\"intro\"><B>Administrative functions:</B><BR>";
		$pString .= "Delete resources, administer styles....</DIV>";
		$pString .= "<DIV CLASS=\"clear\"><HR></DIV>";
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=admin');
		$this->session->setVar('previousLinkName', 'Admin');
		return $pString;
	}
// Administration options:  Resources
	function adminResources()
	{
		global $PHP_SELF;
		include_once("classes/ADMIN.php");
		$admin = new ADMIN($this->db, $this->auth, $this->vars);
		$admin->authorize();
		include_once("classes/CONFIG.php");
		$config = new CONFIG();
		include_once("classes/HTML.php");
		$html = new HTML();
		$config->resourceTypes();
		$this->gatherReport();
		$pString = "<DIV CLASS=\"intro\"><B>Administer Resources:</B><BR>";
		$pString .= "Total:&nbsp;&nbsp;$this->resources</DIV>";
		$pString .= "<DIV CLASS=\"clear\"><HR></DIV>";
		if(!$this->resources)
		{
			$pString .= "There are no resources yet.";
			return $pString;
		}
		$titles = array();
		$types = array();
		$sql = $this->db->select('resource', 'id, title, type', "ORDER BY title");
		$sth = $this->db->query($sql);
		while($row = $this->db->fetchAssoc($sth))
		{
			$titles[$row['id']] = $row['title'];
			$types[$row['id']] = $row['type'];
		}
		$sortedTitles = $this->sortTitles($titles);
		foreach($sortedTitles as $id => $title)
			$assoc[$id] = $title . " (" . $config->types[$types[$id]] . ")";
		$pString .= "<DIV CLASS=\"searchtext\">";
		$pString .= "<FORM METHOD=\"POST\" ACTION=\"" . $PHP_SELF . "\">";
		$pString .= "<INPUT TYPE=\"hidden\" NAME=\"action\" VALUE=\"deleteResource\">";
		$pString .= "<INPUT TYPE=\"hidden\" NAME=\"step\" VALUE=\"1\">";
		$pString .= "<B>Delete Resource:</B><BR>";
		$pString .= $html->selectFBoxValue('id', $assoc, 15);
		$pString .= "<DIV ALIGN=\"right\"><INPUT TYPE=\"submit\" VALUE=\"delete\"></FORM></DIV></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=adminResources');
		$this->session->setVar('previousLinkName', 'Administer Resources');
		return $pString;
	}
// Administration options:  Delete a resource
// Step 1 asks for confirmation, step 2 deletes
	function deleteResource()
	{
		include_once("classes/ADMIN.php");
		$admin = new ADMIN($this->db, $this->auth, $this->vars);
		$admin->authorize();
		include_once("classes/MANIFEST.php");
		$manifest = new MANIFEST(FALSE, $this->db, $this->auth, $this->vars);
		$manifest->deleteResource($this);
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=deleteResource');
		$this->session->setVar('previousLinkName', 'Delete Resources');
		return $admin->deleteResource($this->vars['step'], $this->vars['id']);
	}
// Administration options:  Styles
	function adminStyles()
	{
		include_once("classes/ADMIN.php");
		$admin = new ADMIN($this->db, $this->auth, $this->vars);
		$admin->authorize();
		$pString = "<DIV CLASS=\"intro\"><B>Administer Styles:</B><BR></DIV>";
		$pString .= "<DIV CLASS=\"clear\"><HR></DIV>";
		$pString .= "Coming soon...";
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=adminStyles');
		$this->session->setVar('previousLinkName', 'Administer Styles');
		return $pString;
	}
// Resource front page
	function resource()
	{
		$pString = "<DIV CLASS=\"intro\"><B>Resources:</B><BR>";
		$pString .= "A resource is body of work which can be referenced.  It may be a complete book, an 
			article in a book or a journal article etc. and is therefore analagous to a single entry 
			in a bibliography.<P></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
//		$this->session->resetSession();
		return $pString;
	}
// Search front page
	function searchInit()
	{
		include_once("classes/SEARCH.php");
		$search = new SEARCH($this->db, $this->auth, $this->vars);
		$pString = "<DIV CLASS=\"intro\"><B>Search:</B><BR>";
		$pString .= "You may perform a variety of 
			searches on the resources. All searches are case-<B>in</B>sensitive.</DIV>";
		$pString .= "<DIV CLASS=\"clear\"><HR></DIV>";
		$sql = $this->db->select('summary', 'total_resources');
		if(!$this->db->fetchOne($this->db->query($sql)))
		{
			$pString .= "There are no resources yet.";
		}
		else
		{
			$pString .= $search->init1();
			$pString .= $search->init2();
		}
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=searchInit');
		$this->session->setVar('previousLinkName', 'Search');
		return $pString;
	}
// search the database
	function search()
	{
		include_once("classes/SEARCH.php");
		$search = new SEARCH($this->db, $this->auth, $this->vars);
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=search');
		$this->session->setVar('previousLinkName', 'Search');
		return $search->searchBegin();
	}
// Display form for inputting a new resource
	function newResource()
	{
		include_once("classes/RESOURCE.php");
		$resource = new RESOURCE($this->db, $this->auth, $this->vars);
		if(!$this->session->getVar("write"))
			$this->auth->gracefulExit(TRUE, "You don't have write access.");
		$pString =  $resource->parse();
		$pString .= $this->session->previousLink();
		return $pString;
	}
// Display form for ordering resources
	function orderResources()
	{
		include_once("classes/ORDER.php");
		$order = new ORDER($this->db, $this->auth, $this->vars);
		$pString = "<DIV CLASS=\"intro\"><B>List:</B><BR>";
		$pString .= "You can alphabetically list or extract resources on a variety of parameters.</DIV>";
		$pString .= "<DIV CLASS=\"clear\"><HR></DIV>";
		$sql = $this->db->select('summary', 'total_resources');
		if(!$this->db->fetchOne($this->db->query($sql)))
			return "There are no resources yet.";
		$pString .= $order->init();
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=orderResources');
		$this->session->setVar('previousLinkName', 'List Resources');
		return $pString;
	}
// Display results of ordering resources
	function processOrder()
	{
		include_once("classes/ORDER.php");
		$order = new ORDER($this->db, $this->auth, $this->vars);
		$pString = "<DIV CLASS=\"intro\"><B>List:</B><BR></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
		$pString .= $order->processOrder();
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		if($this->vars["orderMethod"] == 'orderAll')
			$this->session->setVar('previousLink', 'action=processOrder&orderMethod=' . 
				$this->vars["orderMethod"] . '&orderAll=' . $this->vars['orderAll']);
		else
			$this->session->setVar('previousLink', 'action=processOrder&orderMethod=' . 
				$this->vars["orderMethod"]);
		$this->session->setVar('previousLinkName', 'List Resources');
		return $pString;
	}
// Display select box to pick one resource to view
	function listResources()
	{
		global $PHP_SELF;
		include_once("classes/CONFIG.php");
		$config = new CONFIG();
		include_once("classes/HTML.php");
		$html = new HTML();
		$config->resourceTypes();
		$titles = array();
		$types = array();
		$sql = $this->db->select('resource', 'id, title, type', "ORDER BY title");
		$sth = $this->db->query($sql);
		if(!$this->db->numRows($sth))
			return "There are no resources yet.";
		$pString = "<DIV CLASS=\"intro\"><B>View one resource:</B><P></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
		while($row = $this->db->fetchAssoc($sth))
		{
			$titles[$row['id']] = $row['title'];
			$types[$row['id']] = $row['type'];
		}
		$sortedTitles = $this->sortTitles($titles);
		foreach($sortedTitles as $id => $title)
			$assoc[$id] = $title . " (" . $config->types[$types[$id]] . ")";
		$pString .= "<DIV CLASS=\"searchtext\">";
		$pString .= "<FORM METHOD=\"POST\" ACTION=\"" . $PHP_SELF . "\">";
		$pString .= "<INPUT TYPE=\"hidden\" NAME=\"action\" VALUE=\"viewResource\">";
		$pString .= $html->selectFBoxValue('id', $assoc, 15);
		$pString .= "<DIV ALIGN=\"right\"><INPUT TYPE=\"submit\" VALUE=\"proceed\"></FORM></DIV></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=listResources');
		$this->session->setVar('previousLinkName', 'View One Resource');
		return $pString;
	}
// Alphabetically sort the list of titles taking into account prepositions such as 'a', 'an' and 'the' and return 
// a formatted associative array.
	function sortTitles($titles)
	{
// Move prepositions to the rear of the title
		foreach($titles as $id => $title)
		{
			$array = explode(' ', $title);
			$firstWord = array_shift($array);
			$lowercaseWord = strtolower($firstWord);
			if(($lowercaseWord == 'a') || ($lowercaseWord == 'an') || ($lowercaseWord == 'the'))
			{
				$title = implode(' ', $array);
				$title .= ", $firstWord";
			}
			$titles[$id] = $title;
		}
		asort($titles);
		return $titles;
	}
// View full information about one resource
	function viewResource()
	{
		if(!isset($this->vars["id"]))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		$id = $this->vars['id'];
		$pString = "<DIV CLASS=\"intro\"><B>View Resource:</B><BR></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
		list($type, $note, $string) = $this->viewResourceDetails($id);
		$pString .= $string;
		$pString .= $this->viewResourceNote($id, $note);
		$pString .= $this->viewResourceQuotes($type, $id);
		$pString .= $this->viewResourceMusings($type, $id);

// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', "action=viewResource&id=" . $this->vars['id']);
		$this->session->setVar('previousLinkName', 'View One Resource');
		return $pString;
	}
// Display bibliographic details of one resource and links to Notes, Quotes, Musings and Edit Resource
	function displayResource($row, $index, $type = FALSE)
	{
		include_once("classes/STYLE.php");
		$style = new STYLE($this->db);
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		include_once("classes/QUOTES.php");
		$quotes = new QUOTES($this->db);
		include_once("classes/MUSINGS.php");
		$musings = new MUSINGS($this->db);
		include_once("classes/RESOURCE.php");
		$resource = new RESOURCE($this->db, $this->auth, $this->vars);
		global $PHP_SELF;
		if(!$type) $type = $row['type'];
		($index % 2) ? $class = "table2" : $class = "table1";
		$pString = "<DIV CLASS=\"$class\">";
		$pString .= "<DIV CLASS=\"tableLeft\">";
		$pString .= $style->styleFormat($row);
		$pString .= "</DIV>";
		$pString .= "<DIV CLASS=\"tableRight\">";
		$pString .= "<A HREF=\"" . $PHP_SELF . 
			"?action=viewResource&id=" . $row['resourceID'] . "\" TARGET=\"view\">V&nbsp;</A>|";
		$pString .= $notes->displayLink($type, $row['resourceID'], 
			$row['notes'] ? TRUE : FALSE);
		$pString .= $quotes->displayLink($type, $row['resourceID'], 
			$row['num_quotes'] ? TRUE : FALSE);
		$pString .= $musings->displayLink($row['resourceID'], 
			$row['num_musings'] ? TRUE : FALSE);
		if($this->session->getVar('write'))
		{
			$pString .= "|";
			$pString .= $resource->editDisplayLink($row['resourceID']);
		}
		$pString .= "</DIV></DIV><DIV CLASS=\"clear\"></DIV>";
		return $pString;
	}
// view resource notes
	function viewNotes()
	{
		if(!isset($this->vars['id']))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		if(!$pString = $notes->display($this->vars['id']))
			$this->auth->gracefulExit(TRUE, "Resource not found.");
// Write this page link as 'previousLink' to the session.  Failure to write is not disastrous
		$this->session->setVar('previousLink', 'action=viewNotes&id=' . $this->vars['id']);
		$this->session->setVar('previousLinkName', 'View Note');
		return $pString;
	}

// edit a resource note
	function editNote()
	{
		if($this->session->getVar('finaliseNote'))
			$this->auth->gracefulExit(TRUE, 'You have already added/edited this note.');
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		include_once("classes/MANIFEST.php");
		$manifest = new MANIFEST(FALSE, $this->db, $this->auth, $this->vars);
		$manifest->note($notes);
		if(!$pString = $notes->edit($this->vars['id'], $this->vars['notes']))
			$this->auth->gracefulExit(TRUE, "Unable to edit note.");
		return $pString;
	}
// display form for editing a resource note
	function editNoteDisplay()
	{
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		if(!isset($this->vars["id"]))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		$this->session->setVar('previousLink', 'action=editNoteDisplay&id=' . $this->vars['id']);
		$this->session->setVar('previousLinkName', 'Edit Note');
		return $notes->editDisplay($this->vars['id']);
	}
// view resource musings
	function viewMusings()
	{
		if(!isset($this->vars['id']))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		include_once("classes/MUSINGS.php");
		$musings = new MUSINGS($this->db);
		if(!$pString = $musings->display($this->vars['id']))
			$this->auth->gracefulExit(TRUE, "Resource not found.");
		$this->session->setVar('previousLink', 'action=viewMusings');
		$this->session->setVar('previousLinkName', 'View Musings');
		return $pString;
	}
// display form for editing a resource musing
	function editMusingDisplay()
	{
		include_once("classes/MUSINGS.php");
		$musings = new MUSINGS($this->db);
		if(!isset($this->vars["resource_id"]) || !isset($this->vars["id"]))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		$this->session->setVar('previousLink', 'action=editMusingDisplay&id=' . $this->vars['id']);
		$this->session->setVar('previousLinkName', 'Edit Musing');
		return $musings->editDisplay($this->vars["resource_id"], $this->vars['id']);
	}
// add a resource musing
	function addMusing()
	{
// if session variable 'finaliseMusing' is TRUE, user is simply reloading this form
		if($this->session->getVar('finaliseMusing'))
			$this->auth->gracefulExit(TRUE, 'You have already mused.');
		include_once("classes/MUSINGS.php");
		$musings = new MUSINGS($this->db);
		include_once("classes/MANIFEST.php");
		$manifest = new MANIFEST(FALSE, $this->db, $this->auth, $this->vars);
// write input to the session for recall if missing parameters
		if(isset($this->vars['musing']))
			$this->session->setVar('musing', $this->vars['musing']);
		$manifest->musing($musings);
		if(!$pString = $musings->add($this->vars))
			$this->auth->gracefulExit(TRUE, "Unable to add musing.");
		return $pString;
	}
// edit a resource musing
	function editMusing()
	{
// if session variable 'finaliseMusing' is TRUE, user is simply reloading this form
		if($this->session->getVar('finaliseMusing'))
			$this->auth->gracefulExit(TRUE, 'You have already mused.');
		include_once("classes/MUSINGS.php");
		$musings = new MUSINGS($this->db);
		include_once("classes/MANIFEST.php");
		$manifest = new MANIFEST(FALSE, $this->db, $this->auth, $this->vars);
// write input to the session for recall if missing parameters
		if(isset($this->vars['musing']))
			$this->session->setVar('musing', $this->vars['musing']);
		$manifest->musing($musings, TRUE);
		if(!$pString = $musings->edit($this->vars["id"], $this->vars))
			$this->auth->gracefulExit(TRUE, "Unable to edit musing.");
		return $pString;
	}
// view resource quotes
	function viewQuotes()
	{
		if(!isset($this->vars['id']))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		include_once("classes/QUOTES.php");
		$quotes = new QUOTES($this->db);
		if(!$pString = $quotes->display($this->vars['id']))
			$this->auth->gracefulExit(TRUE, "Resource not found.");
		$this->session->setVar('previousLink', 'action=viewQuotes&id=' . $this->vars['id']);
		$this->session->setVar('previousLinkName', 'View Quotes');
		return $pString;
	}
// display form for editing a resource quote
	function editQuoteDisplay()
	{
		include_once("classes/QUOTES.php");
		$quotes = new QUOTES($this->db);
		if(!isset($this->vars["resource_id"]) || !isset($this->vars["id"]))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		return $quotes->editDisplay($this->vars["resource_id"], $this->vars['id']);
		$this->session->setVar('previousLink', 'action=editQuoteDisplay&id=' . $this->vars['id'] . 
			'&resource_id=' . $this->vars["resource_id"]);
		$this->session->setVar('previousLinkName', 'Edit Quote');
	}
// add a resource quote
	function addQuote()
	{
// if session variable 'finaliseQuote' is TRUE, user is simply reloading this form
		if($this->session->getVar('finaliseQuote'))
			$this->auth->gracefulExit(TRUE, 'You have already created this quote.');
		include_once("classes/QUOTES.php");
		$quotes = new QUOTES($this->db);
		include_once("classes/MANIFEST.php");
		$manifest = new MANIFEST(FALSE, $this->db, $this->auth, $this->vars);
// write input to the session for recall if missing parameters
		$array = array("quote", "page", "comment");
		foreach($array as $key)
		{
			if(isset($this->vars[$key]))
				$this->session->setVar($key, $this->vars[$key]);
		}
		$manifest->quote($quotes);
		if(!$pString = $quotes->add($this->vars))
			$this->auth->gracefulExit(TRUE, "Unable to add quote.");
		return $pString;
	}
// edit a resource quote
	function editQuote()
	{
// if session variable 'finaliseQuote' is TRUE, user is simply reloading this form
		if($this->session->getVar('finaliseQuote'))
			$this->auth->gracefulExit(TRUE, 'You have already created this quote.');
		include_once("classes/QUOTES.php");
		$quotes = new QUOTES($this->db);
		include_once("classes/MANIFEST.php");
		$manifest = new MANIFEST(FALSE, $this->db, $this->auth, $this->vars);
// write input to the session for recall if missing parameters
		$array = array("quote", "page", "comment");
		foreach($array as $key)
		{
			if(isset($this->vars[$key]))
				$this->session->setVar($key, $this->vars[$key]);
		}
		$manifest->quote($quotes, TRUE);
		if(!$pString = $quotes->edit($this->vars["resource_id"], $this->vars["id"], $this->vars))
			$this->auth->gracefulExit(TRUE, "Unable to edit quote.");
		return $pString;
	}
// display a resource for editing
	function editResourceDisplay()
	{
		global $PHP_SELF;
		if(!isset($this->vars["id"]) || !$this->vars['id'])
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		$pString = "<DIV CLASS=\"intro\"><B>Edit Resource</B><BR></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
		$pString .= "<DIV class=\"back\" ALIGN=\"right\"><A HREF=\"" . $PHP_SELF . 
				"?action=editResourceDisplay&id=" . $this->vars["id"] . 
				"\">Reload original data</A></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
		include_once("classes/RESOURCE.php");
		$resource = new RESOURCE($this->db, $this->auth, $this->vars);
		$row = $this->editResourceDisplaySql($this->vars['id']);
		$pString .= $resource->editDisplay($row);
		$pString .= "<DIV class=\"back\" ALIGN=\"right\"><A HREF=\"" . $PHP_SELF . 
				"?action=editResourceDisplay&id=" . $this->vars["id"] . 
				"\">Reload original data</A></DIV>";
		$pString .= "<DIV CLASS=\"clear\"></DIV>";
		return $pString;
	}
// create SQL string for editing a resource
	function editResourceDisplaySql($id)
	{
		include_once("classes/CONFIG.php");
		$config = new CONFIG();
		$config->commonSql('all');
		$condition = $config->condition2 . " WHERE resource.id=" . $id;
		$sql = $this->db->select('resource', $config->stmt, $condition);
		$sth = $this->db->query($sql);
		if(!$this->db->numRows($sth))
			$this->auth->gracefulExit(TRUE, "Incorrect parameter.");
		return $this->db->fetchAssoc($sth);
	}
// edit a resource
	function editResource()
	{
		if(!isset($this->vars["id"]))
			$this->auth->gracefulExit(TRUE, "Missing parameter.");
		include_once("classes/RESOURCE.php");
		$resource = new RESOURCE($this->db, $this->auth, $this->vars);
		$pString = $resource->edit($this, $this->vars["id"]);
		$pString .= $this->session->previousLink();
		return $pString;
	}
// format SQL strings for displaying a resource's primary data
// Display the resource details
	function viewResourceDetails($id, $delete = FALSE)
	{
		include_once("classes/STYLE.php");
		$style = new STYLE($this->db);
		include_once("classes/CONFIG.php");
		$config = new CONFIG();
		include_once("classes/RESOURCE.php");
		$resource = new RESOURCE($this->db, $this->auth, $this->vars);
		$config->commonSql('all');
		$config->stmt .= ", num_quotes, num_musings, notes";
		$config->condition .= " LEFT JOIN resource_notes ON resource.id=resource_notes.id 
			WHERE resource.id='$id'";
		$sql = $this->db->select('resource', $config->stmt, $config->condition);
		$row = $this->db->fetchAssoc($this->db->query($sql));
		$pString = "<DIV CLASS=\"table1\">";
		$pString .= "<DIV CLASS=\"tableLeft\">";
		$pString .= $style->styleFormat($row);
		$pString .= "</DIV>";
// If we are displaying this resource in order to delete it, don't display edit links
		if(!$delete)
		{
			$pString .= "<DIV CLASS=\"tableRight\">";
			if($this->session->getVar('write'))
				$pString .= $resource->editDisplayLink($this->vars['id']);
			$pString .= "</DIV>";
		}
		$pString .= "</DIV><DIV CLASS=\"clear\"><HR></DIV>";
		return array($row['resourceID'], $row['notes'], $pString);
	}
// Display the resource note
	function viewResourceNote($id, $note)
	{
		global $PHP_SELF;
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		$index = 0;
		$pString = "<B>Notes:</B><BR>";
		if(!$note)
		{
			$pString .= "No notes yet";
		}
		else
		{
			$pString .= "<DIV CLASS=\"table1\">";
			$pString .= "<DIV CLASS=\"tableLeft\">";
			$pString .= stripslashes($notes->nlReduce($notes->toHtml($note)));
			$pString .= "</DIV>";
			$pString .= "<DIV CLASS=\"tableRight\">";
			if($this->session->getVar('write'))
				$pString .= "<A HREF=\"" . $PHP_SELF . "?action=editNoteDisplay&id=$id\">E</A>&nbsp;";
			$pString .= "</DIV></DIV><DIV CLASS=\"clear\"><HR></DIV>";
		}
// are we allowed to add?
		if(!$note && $this->session->getVar('write'))
		{
			$pString .= "<FORM METHOD=\"POST\" ACTION=\"" . $PHP_SELF . "\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"action\" VALUE=\"editNoteDisplay\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"id\" VALUE=\"$id\">";
			$pString .= "<INPUT TYPE=\"submit\" VALUE=\"add a note\"></FORM><HR>";
		}
		return $pString;
	}
// Display the resource quotes
	function viewResourceQuotes($type, $id)
	{
		global $PHP_SELF;
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		$index = 0;
		$sql = $this->db->select('resource_quotes', 'id, quote, page, comment', "WHERE resource_id=$id 
			ORDER BY page");
		$sth = $this->db->query($sql);
		$pString = "<B>Quotes:</B><BR>";
		if(!$this->db->numRows($sth))
		{
			$pString .= "No quotes yet.";
		}
		while($row = $this->db->fetchAssoc($sth))
		{
			($index++ % 2) ? $class = "table2" : $class = "table1";
			$pString .= "<DIV CLASS=\"$class\">";
			$pString .= "<DIV CLASS=\"tableLeft\">";
			$row['page'] ? $page = $row['page'] : $page = '--';
			$pString .= "Page:&nbsp;<B>" . $page . "</B><BR>";
			$pString .= "<DIV class=\"quote\">\"" . 
				$notes->nlReduce($notes->toHtml(stripslashes($row['quote']))) . "\"</DIV>";
			if($row['comment'])
				$pString .= "<DIV class=\"comment\">" . 
				$notes->nlReduce($notes->toHtml(stripslashes($row['comment']))) . "</DIV>";
			$pString .= "</DIV>";
// are we allowed to edit?
			$pString .= "<DIV CLASS=\"tableRight\">";
			if($this->session->getVar('write'))
				$pString .= "<A HREF=\"" . $PHP_SELF . 
					"?action=editQuoteDisplay&resource_id=$id&id=" . 
					$row['id'] . "\">E</A>&nbsp;";
			$pString .= "</DIV></DIV><DIV CLASS=\"clear\"><HR></DIV>";
		}
// are we allowed to add?
		if($this->session->getVar('write'))
		{
			$pString .= "<FORM METHOD=\"POST\" ACTION=\"" . $PHP_SELF . "\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"action\" VALUE=\"editQuoteDisplay\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"resource_id\" VALUE=\"$id\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"id\" VALUE=\"new\">";
			$pString .= "<INPUT TYPE=\"submit\" VALUE=\"add a quote\"></FORM><HR>";
		}
		return $pString;
	}
// Display the resource musings
	function viewResourceMusings($type, $id)
	{
		global $PHP_SELF;
		include_once("classes/NOTES.php");
		$notes = new NOTES($this->db);
		$index = 0;
		$sql = $this->db->select('musings', 'id, musing', "WHERE resource_id=$id");
		$sth = $this->db->query($sql);
		$pString = "<B>Musings:</B><BR>";
		if(!$this->db->numRows($sth))
		{
			$pString .= "No musings yet.";
		}
		while($row = $this->db->fetchAssoc($sth))
		{
			($index++ % 2) ? $class = "table2" : $class = "table1";
			$pString .= "<DIV CLASS=\"$class\">";
			$pString .= "<DIV CLASS=\"tableLeft\">";
			$pString .= $notes->nlReduce($notes->toHtml(stripslashes($row['musing'])));
			$pString .= "</DIV>";
// are we allowed to edit?
			$pString .= "<DIV CLASS=\"tableRight\">";
			if($this->session->getVar('write'))
				$pString .= "<A HREF=\"" . $PHP_SELF . 
					"?action=editMusingDisplay&resource_id=$id&id=" . 
					$row['id'] . "\">E</A>&nbsp;";
			$pString .= "</DIV></DIV><DIV CLASS=\"clear\"></DIV>";
		}
// are we allowed to add?
		if($this->session->getVar('write'))
		{
			$pString .= "<FORM METHOD=\"POST\" ACTION=\"" . $PHP_SELF . "\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"action\" VALUE=\"editMusingDisplay\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"resource_id\" VALUE=\"$id\">";
			$pString .= "<INPUT TYPE=\"hidden\" NAME=\"id\" VALUE=\"new\">";
			$pString .= "<INPUT TYPE=\"submit\" VALUE=\"muse\"></FORM>";
		}
		return $pString;
	}
// Check config.inc has been set up correctly and set a few defaults
	function checkConfig()
	{
		global $WIKINDX_TITLE;
		global $WIKINDX_DESCRIPTION;
		global $WIKINDX_CSS_STYLE;
		global $WIKINDX_IMAGE;
		global $WIKINDX_IMAGE_HEIGHT;
		global $WIKINDX_IMAGE_WIDTH;
		global $WIKINDX_FRONTPAGE;
		global $WIKINDX_GROUPS;
// don't allow $TITLE not to be set
		if(!isset($WIKINDX_TITLE))
			$this->auth->gracefulExit(FALSE, 'Missing Configuration Parameter: $WIKINDX_TITLE');
		$this->title = $WIKINDX_TITLE;
// don't allow $FRONTPAGE not to be set
		if(!isset($WIKINDX_FRONTPAGE))
			$this->auth->gracefulExit(FALSE, 'Missing Configuration Parameter: $WIKINDX_FRONTPAGE');
// $WIKINDX_GROUPS array must have at least one member keyed with '1'
		if(!isset($WIKINDX_GROUPS) || !sizeof($WIKINDX_GROUPS) || !array_key_exists('1', $WIKINDX_GROUPS))
			$this->auth->gracefulExit(FALSE, 'Missing or Incorrect Configuration 
				Parameter: $WIKINDX_GROUPS');
		$this->frontpage = $WIKINDX_FRONTPAGE;
		if(!isset($WIKINDX_CSS_STYLE) || !$WIKINDX_CSS_STYLE)
		{
			$this->style = "wikindx.css";
		}
		else
		{
			$this->style = $WIKINDX_CSS_STYLE;
		}
		if(!isset($WIKINDX_IMAGE) || !$WIKINDX_IMAGE)
		{
			$this->image = "wikindx.gif";
		}
		else
		{
			$this->image = $WIKINDX_IMAGE;
		}
		$this->imageHeight = $WIKINDX_IMAGE_HEIGHT;
		$this->imageWidth = $WIKINDX_IMAGE_WIDTH;
		if(!isset($WIKINDX_DESCRIPTION) || !$WIKINDX_DESCRIPTION)
		{
			$this->description = "&nbsp;";
		}
		else
		{
			$this->description = $WIKINDX_DESCRIPTION;
		}
	}
// display the right-side bar
	function subMenu()
	{
		print "<DIV CLASS=\"submenu\">";
		print $this->subMenuString;
		print "</DIV>";
	}
// Footer display
	function footer()
	{
		include_once("classes/SCRIPTTIME.php");
		$timer = new SCRIPTTIME();
		global $WIKINDX_DB_QUERIES;
// close mainFrame and main <DIV> started above in constructor;
		print "</DIV>";
		print "<DIV CLASS=\"clear\"></DIV>";
		print "</DIV>";
		print "<DIV CLASS=\"clear\"></DIV>";
// close centralizing table started above in constructor;
		print "<DIV class=\"footer\">
			<A HREF=\"http://sourceforge.net/projects/wikindx/\" TARGET=\"_blank\">wikindx</A>&nbsp;v"
			. $this->version . "&nbsp;~&nbsp;Mark Grimshaw &copy;2004";
		print "&nbsp;&nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;&nbsp;";
		print "Database queries:&nbsp;&nbsp;$WIKINDX_DB_QUERIES";
		print "&nbsp;&nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;&nbsp;";
		$timer->end_time();
		print "Script execution:&nbsp;&nbsp;" . $timer->elapsed_time() . "secs.";
		print "</DIV>";
		print "<DIV CLASS=\"clear\"></DIV>";
// close main encapsulationg DIV (.frame) and HTML
		print "</DIV></BODY></HTML>";
	}
// format submenu strings
	function subMenuFormat($type)
	{
		global $PHP_SELF;
		$default = "Resources:";
		if($this->session->getVar("write"))
			$default .= "&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . "?action=newResource\">New</A>";
		$default .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
			"?action=listResources\">View One</A>";
		$default .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
			"?action=orderResources\">List</A>";
		$default .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
			"?action=searchInit\">Search</A>";
		switch($type)
		{
			case "front":
				$this->gatherReport();
				$this->subMenuString .= "Resources: " . $this->resources;
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Quotes: " . $this->quotes;
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Musings: " . $this->musings;
				break;
			case "bailout":
				$this->subMenuString .= "Resources: " . $this->resources;
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Quotes: " . $this->quotes;
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Musings: " . $this->musings;
				$this->gatherReport();
				break;
			case "resource":
				$this->subMenuString .= $default;
				break;
			case "admin":
				$this->subMenuString .= "Admin:";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=adminResources\">Resources</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=adminStyles\">Styles</A>";
				break;
			case "adminResources":
				$this->subMenuString .= "Admin:";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Resources";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=adminStyles\">Styles</A>";
				break;
			case "deleteResource":
				$this->subMenuString .= "Admin:";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=adminResources\">Resources</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=adminStyles\">Styles</A>";
				break;
			case "adminStyles":
				$this->subMenuString .= "Admin:";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=adminResources\">Resources</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Styles";
				break;
			case "newResource":
				$this->subMenuString .= "Resources:";
				if($this->session->getVar("write"))
				{
					if(isset($this->vars['step']))
						$this->subMenuString .= "&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
						"?action=newResource\">New</A>";
					else
						$this->subMenuString .= "&nbsp;&nbsp;New";
				}
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=listResources\">View One</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=orderResources\">List</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=searchInit\">Search</A>";
				break;
			case "orderResources":
				$this->subMenuString .= "Resources:";
				if($this->session->getVar("write"))
					$this->subMenuString .= "&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=newResource\">New</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=listResources\">View One</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;List";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=searchInit\">Search</A>";
				break;
			case "processOrder":
				$this->subMenuString .= $default;
				break;
			case "listResources":
				$this->subMenuString .= "Resources:";
				if($this->session->getVar("write"))
					$this->subMenuString .= "&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=newResource\">New</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;View One";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=orderResources\">List</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=searchInit\">Search</A>";
				break;
			case "viewResource":
				$this->subMenuString .= $default;
				break;
			case "editResourceDisplay":
				$this->subMenuString .= $default;
				break;
			case "editResource":
				$this->subMenuString .= $default;
				break;
			case "searchInit":
				$this->subMenuString .= "Resources:";
				if($this->session->getVar("write"))
					$this->subMenuString .= "&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=newResource\">New</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=listResources\">View One</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;<A HREF=\"" . $PHP_SELF . 
					"?action=orderResources\">List</A>";
				$this->subMenuString .= "&nbsp;&nbsp;|&nbsp;&nbsp;Search";
				break;
			case "search":
				$this->subMenuString .= $default;
				break;
			case "viewNotes":
				$this->subMenuString .= $default;
				break;
			case "editNote":
				$this->subMenuString .= $default;
				break;
			case "editNoteDisplay":
				$this->subMenuString .= $default;
				break;
			case "viewMusings":
				$this->subMenuString .= $default;
				break;
			case "editMusingDisplay":
				$this->subMenuString .= $default;
				break;
			case "addMusing":
				$this->subMenuString .= $default;
				break;
			case "editMusing":
				$this->subMenuString .= $default;
				break;
			case "viewQuotes":
				$this->subMenuString .= $default;
				break;
			case "editQuoteDisplay":
				$this->subMenuString .= $default;
				break;
			case "addQuote":
				$this->subMenuString .= $default;
				break;
			case "editQuote":
				$this->subMenuString .= $default;
				break;
			default:
				$this->auth->gracefulExit(2, "Incorrect or missing parameters.");
		}
	}
}
?>
